#!/usr/bin/env python3

"""
Basic audio streaming example.

This example shows how to stream the audio data from the TTS engine,
and how to get the WordBoundary events from the engine (which could
be ignored if not needed).

The example streaming_with_subtitles.py shows how to use the
WordBoundary events to create subtitles using SubMaker.
"""

import asyncio
import edge_tts
import subprocess

# TEXT = "Hello World!"
TEXT = "In a quiet village nestled between rolling hills, there lived a young girl named Ella who loved to explore the forest. One day, as she wandered deeper than usual, she stumbled upon an old, overgrown path she had never seen before. Her curiosity piqued, Ella followed the path until she reached a hidden grove, where a magnificent oak tree stood at its center. To her amazement, the tree had a small door at its base."
# TEXT = "在一个宁静的小村庄里，住着一个名叫小明的小男孩。他非常喜欢听爷爷讲故事。一天，小明在村子附近的森林里玩耍时，发现了一只受伤的小狐狸。小狐狸的腿被树枝卡住了，痛苦地叫唤着。小明心疼极了，决定帮助它。他小心翼翼地解开树枝，给小狐狸的腿上涂了一些草药。随后，小明把小狐狸带回了家，用温暖的毛巾包裹住它，并喂了些水。几天后，小狐狸的伤愈合了，它变得活泼健康。"

# VOICE = "en-GB-SoniaNeural"
VOICE = "zh-CN-XiaoyiNeural"
OUTPUT_FILE = "test.mp3"

async def amain() -> None:
    """Main function"""
    communicate = edge_tts.Communicate(TEXT, VOICE)
    with open(OUTPUT_FILE, "wb") as file:
        async for chunk in communicate.stream():
            if chunk["type"] == "audio":
                file.write(chunk["data"])
            elif chunk["type"] == "WordBoundary":
                print(f"WordBoundary: {chunk}")
    # 播放生成的音频文件
    subprocess.run(["mpv", OUTPUT_FILE])

if __name__ == "__main__":
    asyncio.run(amain())