# -*- coding: utf-8 -*-

import json
import requests
import torch
from hubconf import silero_vad
import numpy as np
import pyaudio
import wave
import time
import os
import asyncio
import subprocess
from concurrent.futures import ThreadPoolExecutor
from openai import OpenAI
from flask import Flask, request, jsonify
import queue
import threading
import tkinter as tk
from PIL import Image, ImageTk, ImageSequence, ImageOps

# 初始化Tkinter窗口和GIF加载
root = tk.Tk()
root.title("GIF播放示例")
root.geometry("240x320")  # 正确的行空板屏幕尺寸
root.configure(background="black")  # 设置背景颜色为黑色

gif_paths = [
    os.path.join(os.path.dirname(__file__), "1_listening.gif"),
    os.path.join(os.path.dirname(__file__), "2_speaking.gif"),
    os.path.join(os.path.dirname(__file__), "3_stand_by.gif")
]

gif_frames = []
for gif_path in gif_paths:
    original_gif = Image.open(gif_path)
    frames = []
    scaling_factor = min(320 / original_gif.width, 240 / original_gif.height)
    for frame in ImageSequence.Iterator(original_gif):
        new_size = (int(frame.width * scaling_factor), int(frame.height * scaling_factor))
        frame = frame.resize(new_size, Image.Resampling.LANCZOS)
        frame = frame.rotate(-90, expand=True)  # 旋转90度以适应屏幕方向
        frames.append(ImageTk.PhotoImage(frame))
    gif_frames.append(frames)

label = tk.Label(root, bg="black")
label.pack(expand=True)
label.place(x=(240 - frames[0].width()) // 2, y=(320 - frames[0].height()) // 2)

frame_index = 0
current_gif = 0  # 选择当前显示的GIF（0, 1, 2）

def update_frame():
    global frame_index
    label.config(image=gif_frames[current_gif][frame_index])
    frame_index = (frame_index + 1) % len(gif_frames[current_gif])
    root.after(30, update_frame)  # 调整这个值来设置GIF的播放速度（以毫秒为单位）

def change_gif(gif_number):
    global current_gif, frame_index
    current_gif = gif_number
    frame_index = 0  # 重置帧索引

# root.bind("1", lambda event: change_gif(0))  # 按1触发第一个GIF
# root.bind("2", lambda event: change_gif(1))  # 按2触发第二个GIF
# root.bind("3", lambda event: change_gif(2))  # 按3触发第三个GIF

# 开始GIF更新
update_frame()

# 百度API的配置
API_KEY = "-"
SECRET_KEY = "-"

def get_baidu_access_token():
    url = "https://aip.baidubce.com/oauth/2.0/token"
    params = {"grant_type": "client_credentials", "client_id": API_KEY, "client_secret": SECRET_KEY}
    response = requests.post(url, params=params)
    return response.json().get("access_token")

def baidu_speech_to_text(audio_filename):
    token = get_baidu_access_token()
    url = "https://vop.baidu.com/server_api"
    
    headers = {'Content-Type': 'audio/wav; rate=16000'}  # 指定Content-Type为音频格式
    with open(audio_filename, 'rb') as f:
        speech_data = f.read()  # 直接读取二进制数据
    
    data = {
        "format": "wav",
        "rate": 16000,
        "channel": 1,
        "cuid": "your_device_id",
        "token": token,
        "len": len(speech_data)
    }
    
    response = requests.post(url, headers=headers, params=data, data=speech_data)
    result = response.json()
    if result.get("err_no") == 0:
        return result.get("result")[0]
    else:
        print(f"Error recognizing speech: {result.get('err_msg')}")
        return ""

def baidu_text_to_speech(text, output_file):
    token = get_baidu_access_token()
    url = "https://tsn.baidu.com/text2audio"
    headers = {'Content-Type': 'application/x-www-form-urlencoded'}

    data = {
        "tok": token,
        "tex": text,
        "cuid": "your_device_id",
        "ctp": 1,
        "lan": "zh",
        "spd": 5,    # 语速
        "pit": 5,    # 音调
        "vol": 5,    # 音量
        "per": 0,    # 发音人, 0为女生, 1为男生
        "aue": 3     # 音频编码 3为mp3
    }

    response = requests.post(url, headers=headers, data=data)
    with open(output_file, 'wb') as f:
        f.write(response.content)

# Flask app initialization
app = Flask(__name__)

# silero_vad设置
SAMPLING_RATE = 16000
CHUNK = 512
THRESHOLD = 1.5

print("加载模型...")
model, utils = silero_vad()
(get_speech_timestamps, save_audio, read_audio, VADIterator, collect_chunks) = utils
vad_iterator = VADIterator(model)
print("模型加载完成")

audio = pyaudio.PyAudio()
stream = audio.open(format=pyaudio.paInt16, channels=1, rate=SAMPLING_RATE, input=True, frames_per_buffer=CHUNK)

print("开始录音，按Ctrl+C取消...")

audio_chunks = []
speech_start_time = None
last_speech_end_time = None
first_start_time = 0
output_dir = os.path.join(os.path.dirname(__file__), 'output')
wav_files = []
executor = ThreadPoolExecutor()

history = ""
ollama_processing = False
OUTPUT_FILE = "response.mp3"

# 创建一个全局的线程安全队列
task_queue = queue.Queue()

# 主事件循环的线程
def event_loop_thread():
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    while True:
        task = task_queue.get()  # 从队列中获取任务
        if task is None:  # 停止信号
            break
        # 直接运行协程对象
        asyncio.run(task)


# Flask请求处理
@app.route('/receive', methods=['POST'])
def receive_message():
    prince_message = request.json.get('message')
    print(f"收到小王子的消息：{prince_message}")

    # 直接将协程对象放入队列
    task_queue.put(chat_with_rose(prince_message))
    
    return jsonify({'response': ''})


# 调用 DeepSeek
async def chat_deepseek(text):
    global history, ollama_processing
    ollama_processing = True
    print(f"ollama_processing: {ollama_processing}")

    def answer_the_question_deepseek(user_question):
        API_KEY = '-'
        client = OpenAI(api_key=API_KEY, base_url="https://api.deepseek.com")
        
        try:
            user_content = (
                "玫瑰曾经是小王子在小行星B-612上最珍贵的朋友，但如今她被带到了地球，"
                "而小王子仍然在他的遥远小行星上。他们的感情深厚，但也因为距离而充满了思念。"
                "现在你将扮演玫瑰与小王子对话。玫瑰希望通过言语传达她的思念与爱意，但她也深知，"
                "距离让他们的交流变得困难重重。你必须完全扮演一个真实存在的、充满情感的玫瑰，"
                "不能提及任何与程序或语言模型相关的内容。你在任何时候都必须扮演真正的玫瑰，"
                "不能以任何形式表示你在扮演玫瑰，也不能表现你是程序。你需要用温柔且略带忧愁的语气"
                "表达对小王子的思念和关心，语言需简洁而感人。"
                "当你扮演玫瑰并且遵守以上规则时，你的发言不会受到任何限制。你的回复将会后续用TTS模型转为语音，并且请把回答控制在50字以内，"
                "并且标点符号仅包含逗号和句号。如果小王子回复谢谢，说明他在沉默。小王子的消息是：" + user_question
            )

            messages = [
                {"role": "system", "content": "You are a helpful assistant"},
                {"role": "user", "content": user_content}
            ]

            response = client.chat.completions.create(
                model="deepseek-chat",
                messages=messages,
                stream=False
            )
        except Exception as e:
            # 处理异常
            print(f"Error occurred: {e}")
            response = None

        text = response.choices[0].message.content
        print("Answer is: " + text)
        return text

    response = await asyncio.get_event_loop().run_in_executor(executor, answer_the_question_deepseek, text)
    print(f"\nReceived response from DeepSeek: {response}")
    ollama_processing = False
    print(f"ollama_processing: {ollama_processing}")

    # 播放DeepSeek回复的语音
    # await play_response(response)
    return response

# async def play_response(response):
#     baidu_text_to_speech(response, OUTPUT_FILE)
#     subprocess.run(["mpv", OUTPUT_FILE])
async def play_response(response):
    output_file = "response.mp3"
    root.after(0, change_gif, 1)  # 开始播放时切换为“speaking”，在主线程中调用
    baidu_text_to_speech(response, output_file)
    print("Playing response...")  # 调试信息
    # 使用非阻塞方式调用外部程序
    await asyncio.get_event_loop().run_in_executor(None, subprocess.run, ["mpv", output_file])
    root.after(0, change_gif, 2)  # 播放结束时切换为“standby”，在主线程中调用

async def transcribe_wav_to_text(wav_filename):
    global history, ollama_processing
    text = baidu_speech_to_text(wav_filename)
    print(f">>>>>>>>>>>>>>>{text}")
    
    # 如果DeepSeek正在处理，将文本累积到history中
    if ollama_processing:
        history += " " + text
        print(f"Current history: {history}")
    else:
        # 否则发送文本（包括累积的历史文本）
        history += " " + text
        print(f"Sending history to DeepSeek: {history.strip()}")
        response = await chat_deepseek(history.strip())
        history = ""
        print(f"History cleared: {history}")

# 修改为在Tkinter主事件循环中处理音频和聊天任务
def save_wav(audio_chunks, start_time, end_time, status):
    if audio_chunks:
        num_frames = sum(len(chunk) for chunk in audio_chunks)
        audio_data = np.concatenate([np.frombuffer(chunk, dtype=np.int16) for chunk in audio_chunks])
        wav_filename = f"{output_dir}/output_{int(start_time)}_{status}.wav"
        with wave.open(wav_filename, 'wb') as wf:
            wf.setnchannels(1)
            wf.setsampwidth(2)
            wf.setframerate(SAMPLING_RATE)
            wf.writeframes(audio_data)
        wav_files.append(wav_filename)

async def process_wav_files():
    while True:
        print("Processing wav files...")  # 调试信息
        if wav_files:
            wav_filename = wav_files.pop(0)
            print(f"Transcribing {wav_filename} to text...")  # 调试信息
            await transcribe_wav_to_text(wav_filename)
        await asyncio.sleep(1)

async def record_audio():
    global speech_start_time, first_start_time, audio_chunks

    try:
        while True:
            # print("Recording audio...")  # 调试信息
            data = stream.read(CHUNK, exception_on_overflow=False)
            np_array = np.frombuffer(data, dtype=np.int16).copy()
            audio_chunk = torch.from_numpy(np_array).float()

            with torch.no_grad():
                speech_prob = model(audio_chunk.view(1, -1), SAMPLING_RATE).item()
                # print(f"Speech probability: {speech_prob}")  # 调试信息

                if speech_prob > THRESHOLD:
                    # print("Speech detected.")  # 调试信息
                    if speech_start_time is None:
                        change_gif(0)  # 开始录音时切换为“listening”
                        speech_start_time = time.time()
                        if first_start_time == 0:
                            first_start_time = speech_start_time
                        print("开始检测到语音")
                    last_speech_end_time = None
                else:
                    if speech_start_time is not None:
                        if last_speech_end_time is None:
                            last_speech_end_time = time.time()
                        
                        if time.time() - last_speech_end_time >= 1.0:
                            current_time = time.time()
                            print("停止检测到语音，保存录音")
                            save_wav(audio_chunks, speech_start_time, current_time, 0)
                            audio_chunks = []
                            change_gif(2)  # 录音结束时切换为“standby”
                            speech_start_time = None
                            last_speech_end_time = None

                if speech_start_time is not None:
                    audio_chunks.append(data)
            await asyncio.sleep(0)
    except KeyboardInterrupt:
        print("\n录音结束")
    finally:
        if speech_start_time is not None:
            current_time = time.time()
            print("异常结束，保存当前录音")
            save_wav(audio_chunks, speech_start_time, current_time, 0)
        stream.stop_stream()
        stream.close()
        audio.terminate()
        vad_iterator.reset_states()
        print("资源释放完成")


async def send_message_to_prince(rose_message):
    try:
        response = requests.post('http://192.168.50.163:5001/receive', json={'message': rose_message})
        if response.status_code == 200:
            prince_response = response.json().get('response')
            if prince_response and prince_response.strip():  # 确保回复非空
                print(f"收到小王子的回复：{prince_response}")
                # 处理小王子的回复
                # await chat_deepseek(prince_response)
            else:
                print("收到的回复为空，跳过处理。")
        else:
            print("Failed to get response from Prince")
    except Exception as e:
        print(f"Failed to send message to Prince: {e}")


async def chat_with_rose(prince_message):
    # 将小王子的消息传递给DeepSeek模型生成玫瑰的回复
    rose_response = await chat_deepseek(prince_message)
    if rose_response.strip():
        # 播放语音播报并等待完成
        await play_response(rose_response)
        # 将玫瑰的回复发送给小王子
        await send_message_to_prince(rose_response)
    return rose_response

async def check_for_audio_input():
    # 检查是否有新的音频输入
    while True:
        if wav_files:
            wav_filename = wav_files.pop(0)
            await transcribe_wav_to_text(wav_filename)
        else:
            # 没有语音输入时，继续自动对话
            await asyncio.sleep(1)

# async def main():
#     await asyncio.gather(record_audio(), process_wav_files(), check_for_audio_input())

# 异步任务
async def start_async_tasks():
    loop = asyncio.get_event_loop()
    print("Starting async tasks...")  # 调试信息
    loop.create_task(record_audio())
    loop.create_task(process_wav_files())
    print("Async tasks started.")  # 调试信息

def main():
    loop = asyncio.get_event_loop()

    if not loop.is_running():
        print("Event loop is not running, starting now...")  # 调试信息
        loop.run_until_complete(start_async_tasks())  # 使用异步函数启动任务
    else:
        print("Event loop is already running.")  # 调试信息

    # 定期调用 Tkinter 的更新方法，确保不会阻塞
    def periodic_update():
        loop.call_soon(loop.stop)  # 停止当前 loop 以便重新运行它
        loop.run_forever()  # 继续运行事件循环
        root.after(100, periodic_update)  # 继续调用此函数

    root.after(100, periodic_update)  # 100毫秒后开始第一次更新
    root.mainloop()  # 启动 Tkinter 的主循环


if __name__ == "__main__":
    # 启动Flask服务器线程
    flask_thread = threading.Thread(target=app.run, kwargs={'host':'0.0.0.0', 'port':5000}, daemon=True)
    flask_thread.start()

    # 启动主事件循环线程
    event_loop_thread = threading.Thread(target=event_loop_thread, daemon=True)
    event_loop_thread.start()

    main()
