import tkinter as tk
from PIL import Image, ImageTk, ImageSequence, ImageOps
import os

# 创建主窗口
root = tk.Tk()
root.title("GIF播放示例")
root.geometry("240x320")  # 行空板的正确屏幕尺寸
root.configure(background="black")  # 设置背景颜色为黑色

# GIF动图路径
gif_paths = [
    os.path.join(os.path.dirname(__file__), "1_listening.gif"),
    os.path.join(os.path.dirname(__file__), "2_speaking.gif"),
    os.path.join(os.path.dirname(__file__), "3_stand_by.gif")
]

# 加载三个不同的GIF动图
gif_frames = []
for gif_path in gif_paths:
    original_gif = Image.open(gif_path)
    frames = []
    # 计算缩放比例，使得GIF的最长边等于行空板的最长边（240像素）
    scaling_factor = min(320 / original_gif.width, 240 / original_gif.height)
    
    # 调整GIF大小、旋转并存储每一帧
    for frame in ImageSequence.Iterator(original_gif):
        new_size = (int(frame.width * scaling_factor), int(frame.height * scaling_factor))
        frame = frame.resize(new_size, Image.Resampling.LANCZOS)
        frame = frame.rotate(-90, expand=True)  # 旋转90度以适应屏幕方向
        frames.append(ImageTk.PhotoImage(frame))
    gif_frames.append(frames)

# 初始化标签用于显示GIF，并设置为黑色背景
label = tk.Label(root, bg="black")
label.pack(expand=True)  # 使用expand使标签占满可用空间
label.place(x=(240 - frames[0].width()) // 2, y=(320 - frames[0].height()) // 2)  # 水平/垂直居中

# 当前帧索引
frame_index = 0
current_gif = 0  # 选择当前显示的GIF（0, 1, 2）

# 函数用于更新GIF帧
def update_frame():
    global frame_index
    label.config(image=gif_frames[current_gif][frame_index])
    frame_index = (frame_index + 1) % len(gif_frames[current_gif])
    root.after(30, update_frame)  # 调整这个值来设置GIF的播放速度（以毫秒为单位）

# 函数用于切换GIF
def change_gif(gif_number):
    global current_gif, frame_index
    current_gif = gif_number
    frame_index = 0  # 重置帧索引

# 绑定按键或其他触发条件以更改GIF
root.bind("1", lambda event: change_gif(0))  # 按1触发第一个GIF
root.bind("2", lambda event: change_gif(1))  # 按2触发第二个GIF
root.bind("3", lambda event: change_gif(2))  # 按3触发第三个GIF

# 开始播放GIF
update_frame()

# 启动主事件循环
root.mainloop()
